require File.expand_path('../../helper', __FILE__)
require 'stow/local'

describe Stow::Local do
  before do
    test_dir = File.expand_path('../../', __FILE__)
    @tmp_dir = File.expand_path('tmp', test_dir)
    @fixture_dir = File.expand_path('fixtures', test_dir)
  end
  
  after do
    Dir.foreach(@tmp_dir) do |file|
      next if ['.', '..'].include?(file)
      path = File.join(@tmp_dir, file)
      File.directory?(path) ? FileUtils.rm_rf(path) : File.delete(path)
    end
  end
  
  describe :initialize do
    it "requires a base directory" do
      assert_raises(ArgumentError) { Stow::Local.new }
    end
  end
  
  describe :get do
    before do
      @storage = Stow::Local.new(base: @fixture_dir)
    end
    
    it "can get file" do
      assert_equal Stow::Local::File, @storage.get('file.txt').class
    end
  
    it "can get nested file" do
      assert_equal Stow::Local::File, @storage.get('nested/file.txt').class
    end
  
    it "raises not found error" do
      assert_raises(Stow::NotFoundError) { @storage.get('unknown.txt').class }
    end
    
    it "raises when outsite of base directory" do
      assert_raises(Stow::Local::OutsideBaseError) { @storage.get('../helper.rb') }
    end
  end
  
  describe :put do
    before do
      @source = File.open(File.expand_path('file.txt', @fixture_dir))
      @storage = Stow::Local.new(base: @tmp_dir)
    end
    
    it "can put file" do
      @storage.put(@source, path: 'tester.txt')
      assert_equal 'Hello World', File.read(File.expand_path('tester.txt', @tmp_dir))
    end
    
    it "can put nested file" do
      @storage.put(@source, path: 'inner/tester.txt')
      assert_equal 'Hello World', File.read(File.expand_path('inner/tester.txt', @tmp_dir))
    end
    
    it "can put with string" do
      @storage.put('Hello World', path: 'tester.txt')
      assert_equal 'Hello World', File.read(File.expand_path('tester.txt', @tmp_dir))
    end
    
    it "can put nested with string" do
      @storage.put('Hello World', path: 'inner/tester.txt')
      assert_equal 'Hello World', File.read(File.expand_path('inner/tester.txt', @tmp_dir))
    end
    
    it "can put with stringio" do
      @storage.put(StringIO.new('Hello World'), path: 'tester.txt')
      assert_equal 'Hello World', File.read(File.expand_path('tester.txt', @tmp_dir))
    end
    
    it "can put nested with stringio" do
      @storage.put(StringIO.new('Hello World'), path: 'inner/tester.txt')
      assert_equal 'Hello World', File.read(File.expand_path('inner/tester.txt', @tmp_dir))
    end
    
    it "returns file instance" do
      result = @storage.put(@source, path: 'inner/tester.txt')
      assert_equal Stow::Local::File, result.class
      assert_equal @storage, result.store
      assert_equal 'inner/tester.txt', result.path
    end
    
    it "raises when outsite of base directory" do
      assert_raises(Stow::Local::OutsideBaseError) do
        @storage.put('Howdy', path: '../helper.rb')
      end
    end
  end
  
  describe Stow::Local::File, :path do
    it "can get path" do
      storage = Stow::Local.new(base: @fixture_dir)
      assert_equal 'file.txt', storage.get('file.txt').path
    end
    
    it "can get nested path" do
      storage = Stow::Local.new(base: @fixture_dir)
      assert_equal 'nested/file.txt', storage.get('nested/file.txt').path
    end
  end
  
  describe Stow::Local::File, :body do
    before do
      @path = File.expand_path('existing.txt', @tmp_dir)
      File.open(@path, 'w') { |file| file.write('Existing file') }
      @file = Stow::Local.new(base: @tmp_dir).get('existing.txt')
    end

    it "can get body from string" do
      @file.body = 'Hello World'    
      assert_equal 'Hello World', @file.body
    end

    it "can get body from file" do
      @file.body = File.open(File.expand_path('file.txt', @fixture_dir), 'rb')    
      assert_equal 'Hello World', @file.body
    end

    it "can get body from io" do
      @file.body = StringIO.new('Hello World')    
      assert_equal 'Hello World', @file.body
    end
  end
  
  describe Stow::Local::File, :size do
    before do
      @path = File.expand_path('existing.txt', @tmp_dir)
      File.open(@path, 'w') { |file| file.write('Existing file') }
      @file = Stow::Local.new(base: @tmp_dir).get('existing.txt')
    end
    
    it "can get size in bytes when persisted" do
      assert_equal 13, @file.size
    end

    it "can get size in bytes from supplied file" do
      File.open(File.expand_path('file.txt', @fixture_dir), 'rb') do |file|
        @file.body = file
      end
      assert_equal 11, @file.size
    end
    
    it "can get size in bytes from supplied string" do
      @file.body = 'Something else'
      assert_equal 14, @file.size
    end
    
    it "can get size in bytes from supplied io" do
      @file.body = StringIO.new('An IO')
      assert_equal 5, @file.size
    end
  end
  
  describe Stow::Local::File, :save do
    before do
      @path = File.expand_path('nested/existing.txt', @tmp_dir)
      Dir.mkdir(File.dirname(@path))
      File.open(@path, 'w') { |file| file.write('Existing file') }
      @file = Stow::Local.new(base: @tmp_dir).get('nested/existing.txt')
    end

    it "does noting when persisted already" do
      time = File.mtime(@path)
      @file.save
      assert_equal time, File.mtime(@path)
    end
  
    it "can save with string" do
      @file.body = 'Hello World'
      @file.save
      assert_equal 'Hello World', File.read(@path)
    end

    it "can save with file" do
      File.open(File.expand_path('file.txt', @fixture_dir), 'rb') do |file|
        @file.body = file
      end
      @file.save
      assert_equal 'Hello World', File.read(@path)
    end

    it "can save with io" do
      @file.body = StringIO.new('Hello World')
      @file.save
      assert_equal 'Hello World', File.read(@path)
    end
    
    it "can save with something that has a path e.g. Rails upload" do
      @file.body = stub(path: File.expand_path('file.txt', @fixture_dir))
      @file.save
      assert_equal 'Hello World', File.read(@path)
    end
  end
  
  describe Stow::Local::File, :update do
    before do
      @path = File.expand_path('nested/existing.txt', @tmp_dir)
      Dir.mkdir(File.dirname(@path))
      File.open(@path, 'w') { |file| file.write('Existing file') }
      @file = Stow::Local.new(base: @tmp_dir).get('nested/existing.txt')
    end
  
    it "can update body with string" do
      @file.update(body: 'Hello World')
      assert_equal 'Hello World', File.read(@path)
    end

    it "can update body with file" do
      @file.update(body: File.open(File.expand_path('file.txt', @fixture_dir), 'rb'))
      assert_equal 'Hello World', File.read(@path)
    end

    it "can update body with io" do
      @file.update(body: StringIO.new('Hello World'))
      assert_equal 'Hello World', File.read(@path)
    end
  end
  
  describe Stow::Local::File, :destroy do
    before do
      @path = File.expand_path('nested/existing.txt', @tmp_dir)
      Dir.mkdir(File.dirname(@path))
      File.open(@path, 'w') { |file| file.write('Existing file') }
      @file = Stow::Local.new(base: @tmp_dir).get('nested/existing.txt')
    end
    
    it "can delete file" do
      @file.destroy
      assert_equal false, File.exist?(@path)
      assert_equal nil, @file.body
    end
    
    it "removes empty directories" do
      @file.destroy
      assert_equal false, File.directory?(File.expand_path('nested', @tmp_dir))
    end
    
    it "leaves populated directories" do
      FileUtils.touch(File.expand_path('nested/another.txt', @tmp_dir))
      @file.destroy
      assert_equal true, File.directory?(File.expand_path('nested', @tmp_dir))
    end
  end
  
  describe Stow::Local::File, :copy do
    before do
      @path = File.expand_path('nested/existing.txt', @tmp_dir)
      Dir.mkdir(File.dirname(@path))
      File.open(@path, 'w') { |file| file.write('Existing file') }
      @file = Stow::Local.new(base: @tmp_dir).get('nested/existing.txt')
    end
    
    it "requires a path" do
      assert_raises(ArgumentError) { @file.copy }
    end
    
    it "copies file" do
      @file.copy path: 'another/file.txt'
      assert_equal 'Existing file', File.read(File.join(@tmp_dir, 'another/file.txt'))
    end
  end
end